use std::path::PathBuf;

use gtk::prelude::*;
use relm4::prelude::*;

use crate::ui::{mcdu::McduOutput, sheet_listing::SheetPressedMessage};

use super::{
    mcdu::McduModel,
    sheet_listing::{SheetListingInput, SheetListingModel},
};

pub struct AppModel {
    mcdu: Controller<McduModel>,
    sheet_listing: Controller<SheetListingModel>,
}

#[derive(Debug)]
pub enum AppInput {
    SearchStarted(String),
    SheetPressed(PathBuf),
}

#[relm4::component(pub)]
impl SimpleComponent for AppModel {
    type Input = AppInput;
    type Output = ();
    type Init = PathBuf;

    view! {
        #[root]
        gtk::Window{
            set_default_width: 300,
            set_default_height: 300,
            set_title: Some("Play music!!!"),
            set_maximized: true,
            gtk::Box {
                set_orientation: gtk::Orientation::Horizontal,
                gtk::Box {
                    set_orientation: gtk::Orientation::Vertical,
                    set_hexpand: true,
                    gtk::ScrolledWindow {
                        model.sheet_listing.widget(),
                        set_vexpand: true,
                        set_hexpand: true,
                        // set_hscrollbar_policy: PolicyType::Never,
                    },
                },
                model.mcdu.widget(),
            }
        }
    }

    fn init(
        path: Self::Init,
        window: &Self::Root,
        sender: ComponentSender<Self>,
    ) -> relm4::ComponentParts<Self> {
        relm4_icons::initialize_icons();

        let mcdu = McduModel::builder()
            .launch(())
            .forward(sender.input_sender(), |response| match response {
                McduOutput::SearchStarted(query) => AppInput::SearchStarted(query),
            });

        let sheet_listing =
            SheetListingModel::builder()
                .launch(path)
                .forward(sender.input_sender(), |response| match response {
                    SheetPressedMessage::SheetPressed(path) => AppInput::SheetPressed(path),
                });

        let model = AppModel {
            mcdu,
            sheet_listing,
        };

        let widgets = view_output!();

        ComponentParts { model, widgets }
    }

    fn update(&mut self, message: Self::Input, _sender: ComponentSender<Self>) {
        match message {
            AppInput::SheetPressed(sheet) => opener::open(sheet).unwrap(),
            AppInput::SearchStarted(query) => self.sheet_listing.emit(SheetListingInput { query }),
        }
    }
}
